@@ -5,7 +5,7 @@
 //! * [InMemoryClient]: A client which keeps data in memory and never fails
 //! * [failing_client_simulator]: A module containing a client with data kept in memory
 //! which can fail and cause panics.
-use std::ops::RangeInclusive;
+use std::{convert::Infallible, fmt, ops::RangeInclusive, str::FromStr};
 
 use futures::future::BoxFuture;
 
@@ -29,10 +29,10 @@ pub enum DownloadSpec {
 impl DownloadSpec {
     /// Returns a value for an  `HTTP-Range` header with bytes as the unit
     /// if the variant is [DownloadSpec::Range]
-    pub fn http_range_value(&self) -> Option<String> {
+    pub fn http_bytes_range_value(&self) -> Option<String> {
         match self {
             DownloadSpec::Complete => None,
-            DownloadSpec::Range(r) => Some(r.http_range_value()),
+            DownloadSpec::Range(r) => Some(r.http_bytes_range_value()),
         }
     }
 
@@ -45,6 +45,15 @@ impl DownloadSpec {
     }
 }
 
+impl fmt::Display for DownloadSpec {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            DownloadSpec::Complete => write!(f, "[..]"),
+            DownloadSpec::Range(r) => r.fmt(f),
+        }
+    }
+}
+
 impl From<InclusiveRange> for DownloadSpec {
     fn from(r: InclusiveRange) -> Self {
         Self::Range(r)
@@ -81,14 +90,22 @@ pub trait CondowClient: Clone + Send + Sync + 'static {
 
 /// A location usable for testing.
 #[derive(Debug, Clone, Copy)]
-pub struct NoLocation;
+pub struct IgnoreLocation;
 
-impl std::fmt::Display for NoLocation {
+impl std::fmt::Display for IgnoreLocation {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         write!(f, "<no location>")
     }
 }
 
+impl FromStr for IgnoreLocation {
+    type Err = Infallible;
+
+    fn from_str(_: &str) -> Result<Self, Self::Err> {
+        Ok(IgnoreLocation)
+    }
+}
+
 mod in_memory {
     use std::{marker::PhantomData, sync::Arc};
 
@@ -104,14 +121,15 @@ mod in_memory {
         future::{self, BoxFuture, FutureExt},
         stream,
     };
+    use tracing::trace;
 
-    use super::{CondowClient, DownloadSpec, NoLocation};
+    use super::{CondowClient, DownloadSpec, IgnoreLocation};
 
-    /// Holds the BLOB in memory as owned data.
+    /// Holds the BLOB in memory as owned or static data.
     ///
     /// Use for testing.
     #[derive(Clone)]
-    pub struct InMemoryClient<L = NoLocation> {
+    pub struct InMemoryClient<L = IgnoreLocation> {
         blob: Blob,
         chunk_size: usize,
         _location: PhantomData<L>,
@@ -171,6 +189,8 @@ mod in_memory {
             &self,
             _location: Self::Location,
         ) -> BoxFuture<'static, Result<u64, CondowError>> {
+            trace!("in-memory-client: get_size");
+
             futures::future::ready(Ok(self.blob.len() as u64)).boxed()
         }
 
@@ -179,6 +199,8 @@ mod in_memory {
             _location: Self::Location,
             spec: DownloadSpec,
         ) -> BoxFuture<'static, Result<(BytesStream, BytesHint), CondowError>> {
+            trace!("in-memory-client: download");
+
             download(&self.blob.as_slice(), self.chunk_size, spec)
         }
     }
@@ -337,6 +359,7 @@ pub mod failing_client_simulator {
 
     use bytes::Bytes;
     use futures::{future, lock::Mutex, task, FutureExt, Stream, StreamExt};
+    use tracing::trace;
 
     use crate::{
         condow_client::{CondowClient, DownloadSpec},
@@ -346,7 +369,7 @@ pub mod failing_client_simulator {
         Condow, InclusiveRange,
     };
 
-    pub use super::NoLocation;
+    pub use super::IgnoreLocation;
 
     /// A builder for a [FailingClientSimulator]
     pub struct FailingClientSimulatorBuilder {
@@ -400,7 +423,7 @@ pub mod failing_client_simulator {
         /// If `chunk_size` is 0.
         pub fn chunk_size(mut self, chunk_size: usize) -> Self {
             if chunk_size == 0 {
-                panic!("chun size must be greater than 0")
+                panic!("chunk size must be greater than 0")
             }
 
             self.chunk_size = chunk_size;
@@ -431,7 +454,7 @@ pub mod failing_client_simulator {
     ///
     /// Clones will share the responses to be played back
     #[derive(Clone)]
-    pub struct FailingClientSimulator<L = NoLocation> {
+    pub struct FailingClientSimulator<L = IgnoreLocation> {
         blob: Blob,
         responses: Arc<Mutex<vec::IntoIter<ResponseBehaviour>>>,
         chunk_size: usize,
@@ -467,6 +490,7 @@ pub mod failing_client_simulator {
             &self,
             _location: Self::Location,
         ) -> futures::future::BoxFuture<'static, Result<u64, CondowError>> {
+            trace!("failing-client-simulator: get_size");
             future::ready(Ok(self.blob.len() as u64)).boxed()
         }
 
@@ -476,6 +500,7 @@ pub mod failing_client_simulator {
             spec: DownloadSpec,
         ) -> futures::future::BoxFuture<'static, Result<(BytesStream, BytesHint), CondowError>>
         {
+            trace!("failing-client-simulator: download");
             let me = self.clone();
             let range_incl = match spec {
                 DownloadSpec::Range(r) => r,
@@ -531,7 +556,10 @@ pub mod failing_client_simulator {
                             blob: me.blob,
                             next: start,
                             end_excl,
-                            error: Some(IoError(error_offset.to_string())),
+                            error: Some(ErrorAction::Err(format!(
+                                "stream error at {}",
+                                error_offset
+                            ))),
                             chunk_size: me.chunk_size,
                         };
                         Ok((stream.boxed(), bytes_hint))
@@ -551,7 +579,17 @@ pub mod failing_client_simulator {
                             );
                         }
 
-                        panic!("panic at byte {} of range {}", panic_offset, range_incl);
+                        let stream = BytesStreamWithError {
+                            blob: me.blob,
+                            next: start,
+                            end_excl,
+                            error: Some(ErrorAction::Panic(format!(
+                                "panic at byte {} of range {}",
+                                panic_offset, range_incl
+                            ))),
+                            chunk_size: me.chunk_size,
+                        };
+                        Ok((stream.boxed(), bytes_hint))
                     }
                 }
             }
@@ -830,11 +868,16 @@ pub mod failing_client_simulator {
         SuccessWithStreamPanic(usize),
     }
 
+    pub enum ErrorAction {
+        Err(String),
+        Panic(String),
+    }
+
     struct BytesStreamWithError {
         blob: Blob,
         next: usize,
         end_excl: usize,
-        error: Option<IoError>,
+        error: Option<ErrorAction>,
         chunk_size: usize,
     }
 
@@ -846,8 +889,11 @@ pub mod failing_client_simulator {
             _cx: &mut std::task::Context<'_>,
         ) -> task::Poll<Option<Self::Item>> {
             if self.next == self.end_excl || self.chunk_size == 0 {
-                if let Some(err) = self.error.take() {
-                    return task::Poll::Ready(Some(Err(err)));
+                if let Some(error_action) = self.error.take() {
+                    match error_action {
+                        ErrorAction::Err(msg) => return task::Poll::Ready(Some(Err(IoError(msg)))),
+                        ErrorAction::Panic(msg) => panic!("{}", msg),
+                    }
                 } else {
                     return task::Poll::Ready(None);
                 }
@@ -1182,7 +1228,7 @@ pub mod failing_client_simulator {
             client: &FailingClientSimulator,
             range: R,
         ) -> Result<Result<Vec<u8>, Vec<u8>>, CondowError> {
-            let (mut stream, _bytes_hint) = client.download(NoLocation, range.into()).await?;
+            let (mut stream, _bytes_hint) = client.download(IgnoreLocation, range.into()).await?;
 
             let mut received = Vec::new();
 
@@ -1354,7 +1400,7 @@ pub mod failing_client_simulator {
                 next: range.start,
                 end_excl: range.end,
                 error: if err {
-                    Some(IoError("bang!".to_string()))
+                    Some(ErrorAction::Err("bang!".to_string()))
                 } else {
                     None
                 },
